==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/extractAlleles.R <==
## functions for extracting allele levels, allele pairs, and allele codes
extractAlleles <- function(object, which=c(1,2), codes=FALSE,
                           allow.partial.missing=FALSE,
                           marker,
                           obs)
          {
            if(missing(marker))
              marker <- markerNames(object)
            else if(is.numeric(marker))
              marker <- markerNames(object)[marker]

            if(missing(obs))
              obs <- TRUE
            
            miss <- missingCodes(object, marker, drop=FALSE)

            tt   <- transTables(object, marker, drop=FALSE)

            # apply NA values
            makemiss <- function(x, miss)
              {

                x[is.na(x) | x %in% miss] <- NA
                if(!allow.partial.missing)
                  {
                    flag <- apply( x, 1, function(X) any(is.na(X) | X %in% NA))
                    x[flag,] <- NA
                  }
                else
                  {
                    flag <- apply( x[,-1], 1, function(X) all(is.na(X) | X %in% NA))
                    x[flag,] <- NA
                  }
                  
                x
              }
            
            ## convert to numeric values & apply NA codes
            conv <- function(mat)
              { 
                alleles <- na.omit(unique(c(mat[,-1])))
                ttcodes <- apply(mat, 2, function(x)match(x,alleles))
                ttcodes[,1] <- 1:nrow(ttcodes)
                
                if(!allow.partial.missing)
                  {
                    flag <- apply( ttcodes[,-1], 1, function(x) any(is.na(x)))
                    ttcodes[flag,] <- NA
                  }
                else
                  {
                    flag <- apply( ttcodes, 1, function(x) all(is.na(x[-1])))
                    ttcodes[flag,] <- NA
                  }
                  
                ttcodes[!is.na(ttcodes[,1]),1] <- 1:sum(!is.na(ttcodes[,1]))
                ttcodes
              }

            for(m in marker)
              {
                  tt[[m]] <- makemiss(tt[[m]],miss[[m]])
                  if(codes) 
                    tt[[m]] <- conv(tt[[m]])
              }

            tnames <- marker
            tnlen <- length(marker)

            f <- function(x) tt[[x]][
                                     object@callCodes[x,obs],
                                     which+1,drop=FALSE
                                     ]
                                     

            retval <- do.call("cbind", lapply(tnames, f)  )
            
            if(length(which)>1)
              {
                suffix <- paste(  '.', which, sep='')
                suffix[suffix==".0"] <- ""
                colnames(retval) <- paste( rep(tnames, rep(length(which), tnlen)), 
                                          suffix, sep='')
              }
            else
              colnames(retval) <- tnames

            rownames(retval) <- colnames(object@callCodes[,obs])
            
            retval
          }

## Extract unique marker alleles
setGeneric("alleleLevels", function(object, ...)
           standardGeneric("alleleLevels"))


setMethod("alleleLevels", "geneSet", function(object, marker)
          {
            if(missing(marker)) marker <- 1:nmarker(object)
            if(is.numeric(marker)) marker <- markerNames(object)[marker]

            retval <- lapply(marker,
                             function(x)
                             {
                               tt <- transTables(object, x, drop=TRUE )
                               mc <- missingCodes(object, x, drop=TRUE)
                               
                               retval <- unique(c(tt[,-1]))
                               retval[! retval %in% mc ] 
                             }
                             )
            names(retval) <- marker
            retval
          }
          )

==> GeneticsBase_1.6.0.tar.gz: GeneticsBase/R/extractAlleles.R <==
## functions for extracting allele levels, allele pairs, and allele codes
extractAlleles <- function(object, which=c(1,2), codes=FALSE,
                           allow.partial.missing=FALSE,
                           marker,
                           obs)
          {
            if(missing(marker))
              marker <- markerNames(object)
            else if(is.numeric(marker))
              marker <- markerNames(object)[marker]

            if(missing(obs))
              obs <- TRUE
            
            miss <- missingCodes(object, marker, drop=FALSE)

            tt   <- transTables(object, marker, drop=FALSE)

            # apply NA values
            makemiss <- function(x, miss)
              {

                x[is.na(x) | x %in% miss] <- NA
                if(!allow.partial.missing)
                  {
                    flag <- apply( x, 1, function(X) any(is.na(X) | X %in% NA))
                    x[flag,] <- NA
                  }
                else
                  {
                    flag <- apply( x[,-1, drop=FALSE], 1, function(X) all(is.na(X) | X %in% NA))
                    x[flag,] <- NA
                  }
                  
                x
              }
            
            ## convert to numeric values & apply NA codes
            conv <- function(mat)
              { 
                alleles <- na.omit(unique(c(mat[,-1, drop=FALSE])))
                ttcodes <- apply(mat, 2, function(x) match(x,alleles))
                if(!is.matrix(ttcodes)) ttcodes <- matrix(ttcodes, nrow=1, ncol=length(ttcodes) )
                ttcodes[,1] <- 1:nrow(ttcodes)
                
                if(!allow.partial.missing)
                  {
                    flag <- apply( ttcodes[,-1, drop=FALSE], 1, function(x) any(is.na(x)))
                    ttcodes[flag,] <- NA
                  }
                else
                  {
                    flag <- apply( ttcodes, 1, function(x) all(is.na(x[-1])))
                    ttcodes[flag,] <- NA
                  }
                  
                ttcodes[!is.na(ttcodes[,1]),1] <- 1:sum(!is.na(ttcodes[,1]))
                ttcodes
              }

            for(m in marker)
              {
                  tt[[m]] <- makemiss(tt[[m]],miss[[m]])
                  if(codes)
                    tt[[m]] <- conv(tt[[m]])
              }

            tnames <- marker
            tnlen <- length(marker)

            f <- function(x) tt[[x]][
                                     object@callCodes[x,obs],
                                     which+1,drop=FALSE
                                     ]
                                     

            retval <- do.call("cbind", lapply(tnames, f)  )
            
            if(length(which)>1)
              {
                suffix <- paste(  '.', which, sep='')
                suffix[suffix==".0"] <- ""
                colnames(retval) <- paste( rep(tnames, rep(length(which), tnlen)), 
                                          suffix, sep='')
              }
            else
              colnames(retval) <- tnames

            rownames(retval) <- colnames(object@callCodes[,obs])
            
            retval
          }

## Extract unique marker alleles
setGeneric("alleleLevels", function(object, ...)
           standardGeneric("alleleLevels"))


setMethod("alleleLevels", "geneSet", function(object, marker)
          {
            if(missing(marker)) marker <- 1:nmarker(object)
            if(is.numeric(marker)) marker <- markerNames(object)[marker]

            retval <- lapply(marker,
                             function(x)
                             {
                               tt <- transTables(object, x, drop=TRUE )
                               mc <- missingCodes(object, x, drop=TRUE)
                               
                               retval <- unique(c(tt[,-1, drop=FALSE]))
                               retval[! retval %in% mc ] 
                             }
                             )
            names(retval) <- marker
            retval
          }
          )
